package de.jottyfan.timetrack.modules.done;

import static de.jottyfan.timetrack.db.done.Tables.V_CURRENT_OVERTIME;
import static de.jottyfan.timetrack.db.done.Tables.V_DAY;
import static de.jottyfan.timetrack.db.profile.Tables.T_LOGIN;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalTime;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.Field;
import org.jooq.Record6;
import org.jooq.SelectConditionStep;
import org.jooq.impl.DSL;
import org.jooq.types.YearToSecond;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.timetrack.modules.done.model.DaysumBean;

/**
 * 
 * @author jotty
 * 
 */
@Repository
public class DoneRepository {
	private static final Logger LOGGER = LogManager.getLogger(DoneRepository.class);

	@Autowired
	private DSLContext jooq;

	public DaysumBean getDaysum(LocalDate day, String login) {
		Field<LocalTime> WORKTIME = DSL.field("worktime", LocalTime.class);
		Field<LocalTime> BREAKTIME = DSL.field("breaktime", LocalTime.class);
		
		SelectConditionStep<Record6<LocalTime, LocalTime, LocalTime, LocalTime, YearToSecond, LocalTime>> sql = jooq
		// @formatter:off
			.select(V_DAY.STARTTIME,
					    V_DAY.ENDTIME,
					    V_DAY.WORKTIME.cast(LocalTime.class).as(WORKTIME),
					    V_DAY.BREAKTIME.cast(LocalTime.class).as(BREAKTIME),
					    V_DAY.DAY_OVERTIME,
					    V_CURRENT_OVERTIME.OVERTIME.plus(DSL.coalesce(V_DAY.DAY_OVERTIME, YearToSecond.valueOf(0))).as(V_CURRENT_OVERTIME.OVERTIME))
			.from(V_DAY)
			.innerJoin(T_LOGIN).on(T_LOGIN.PK.eq(V_DAY.FK_LOGIN))
			.leftJoin(V_CURRENT_OVERTIME).on(V_CURRENT_OVERTIME.FK_LOGIN.eq(V_DAY.FK_LOGIN))
			.where(V_DAY.DAY.eq(day))
			.and(T_LOGIN.LOGIN.eq(login));
		// @formatter:on
		LOGGER.trace(sql);
		Record6<LocalTime, LocalTime, LocalTime, LocalTime, YearToSecond, LocalTime> r = sql.fetchOne();
		if (r == null) {
			return null;
		} else {
			DaysumBean bean = new DaysumBean();
			bean.setDaytimeFrom(r.get(V_DAY.STARTTIME));
			bean.setDaytimeUntil(r.get(V_DAY.ENDTIME));
			bean.setDayworktime(r.get(WORKTIME));
			bean.setBreaks(r.get(BREAKTIME));
			YearToSecond dayOvertime = r.get(V_DAY.DAY_OVERTIME);
			Duration dayOvertimeDuration = dayOvertime == null ? null : dayOvertime.toDuration();
			String dayOvertimeString = dayOvertimeDuration == null ? null : String.format("%3d:%2d", dayOvertimeDuration.toHours(), Math.abs(dayOvertimeDuration.toMinutes() % 60));
			bean.setDayOvertime(dayOvertimeString == null ? "?" : dayOvertimeString);
			bean.setTotalOvertime(r.get(V_CURRENT_OVERTIME.OVERTIME));
			return bean;
		}
	}
}
